//
//  BlockSpinsParamsPanelCtl.m
//  IsingCocoa
//
//  Created by Cameron Hayne on 08/05/05.
//  Copyright Hayne of Tintagel 2005
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

#import "BlockSpinsParamsPanelCtl.h"
#import "IsingWindowCtl.h"
#import "IsingModel.h"
#import "IsingSpinView.h"
#import "BlockSpinsWin.h"

@implementation BlockSpinsParamsPanelCtl

- (id)initForWindowCtl:(IsingWindowCtl *)aWindowCtl
{
    self = [super initWithWindowNibName: @"BlockSpinsParamsPanel"];
    if (self == nil)
    {
        return nil;
    }
    
    //NSLog(@"BlockSpinsParamsPanelCtl.init");
    isingWindowCtl = aWindowCtl;
    mainWindow = [isingWindowCtl window];
    spinView = [isingWindowCtl spinView];
    model = [isingWindowCtl model];
    
    [self window]; // force loading of the NIB
    return self;
}

- (void)dealloc
{
    //NSLog(@"BlockSpinsParamsPanelCtl.dealloc");
    [super dealloc];
}

- (void)windowDidLoad
{
    //NSLog(@"BlockSpinsParamsPanelCtl.windowDidLoad");
    BOOL defaultBlocksAreSquare = YES;
    int defaultBlockWidth = 3;
    int defaultBlockHeight = 3;
    
    [squareBlocksChk setState: defaultBlocksAreSquare];
    [self squareChkAction: squareBlocksChk];
    [widthPopup selectItemAtIndex: [widthPopup indexOfItemWithTag: defaultBlockWidth]];
    [heightPopup selectItemAtIndex: [heightPopup indexOfItemWithTag: defaultBlockHeight]];
    
    [NSApp beginSheet: sheet
       modalForWindow: mainWindow
        modalDelegate: nil
       didEndSelector: NULL
          contextInfo: nil];
    
    // We need to retain the BlockSpinsParamsPanelCtl object since it is released
    // in IsingWindowCtl and we need it to stick around until the sheet is closed
    [self retain];
}

- (IBAction)squareChkAction:(id)sender
{
    BOOL square = ([sender state] == NSOnState);
    if (square)
    {
        [heightPopup setEnabled: NO];
        int blockWidth = [widthPopup selectedTag];
        [heightPopup selectItemAtIndex: [heightPopup indexOfItemWithTag: blockWidth]];
    }
    else
    {
        [heightPopup setEnabled: YES];
    }
}

- (IBAction)widthPopupAction:(id)sender
{
    if ([squareBlocksChk state] == NSOnState)
    {
        int blockWidth = [widthPopup selectedTag];
        [heightPopup selectItemAtIndex: [heightPopup indexOfItemWithTag: blockWidth]];
    }
}

- (IBAction)closeSheet:(id)sender
{
    [sheet orderOut: nil];
    [NSApp endSheet: sheet];
    
    // We no longer need the BlockSpinsParamsPanelCtl object
    [self autorelease];
    
    if (sender != okBtn)
    {
        // user cancelled
        return;
    }
    
    int blockWidth = [widthPopup selectedTag];
    int blockHeight = [heightPopup selectedTag];
    IsingSize blockSize = makeIsingSize(blockWidth, blockHeight);

    BlockSpinsWin *blockSpinsWin = [[[BlockSpinsWin alloc]
                                        initWithSpinsFrameSize: [spinView frame].size
                                                    mainWindow: mainWindow
                                                     spinsSize: [model size]
                                                     blockSize: blockSize]
                                        autorelease];

    [[isingWindowCtl document] addWindowController: blockSpinsWin];
    [isingWindowCtl addIsingView: blockSpinsWin];
    // note that the IsingViewCtl adds an observer so it gets notified when
    // the BlockSpinsWin gets closed so as to auto-remove the view from its list
}

@end
