//
//  IsingControllerToolbar.m
//  IsingCocoa
//
//  Created by Cameron Hayne on Thu Nov 06 2003.
//  Copyright Hayne of Tintagel 2005
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

#import "IsingToolbar.h"
#import "IsingModel.h"

static NSString *IsingWindowTB = @"IsingWindow Toolbar";
static NSString *StatisticsTBI = @"Statistics ToolbarItem";
static NSString *PauseTBI = @"Pause ToolbarItem";
static NSString *FieldTBI = @"Field ToolbarItem";
static NSString *UpdateRateTBI = @"UpdateRate ToolbarItem";

@implementation IsingWindowCtl (IsingToolbar)

- (void)setupToolbar
{
    NSToolbar *toolbar = [[NSToolbar alloc] initWithIdentifier: IsingWindowTB];
    [toolbar setDelegate:self];
    [toolbar setSizeMode:NSToolbarSizeModeSmall];
    [toolbar setAllowsUserCustomization:YES];
    [toolbar setAutosavesConfiguration:YES];
    [[self window] setToolbar:toolbar];
    [toolbar autorelease];
}

- (void) setupPauseToolbarItem:(NSToolbarItem *)item forState:(BOOL)updateIsActive
{
    if (updateIsActive)
    {
        [item setLabel: @"Pause"];
        [item setToolTip: @"Pause the simulation"];
        [item setImage:[NSImage imageNamed: @"pause"]];
    }
    else
    {
        [item setLabel: @"Resume"];
        [item setToolTip: @"Resume the simulation"];
        [item setImage:[NSImage imageNamed: @"resume"]];
    }
}

- (void) setupFieldToolbarItem:(NSToolbarItem *)item forState:(BOOL)fieldIsPresent
{
    if (fieldIsPresent)
    {
        [item setLabel: @"Disable Field"];
        [item setToolTip: @"Disable the ambient magnetic field"];
        [item setImage:[NSImage imageNamed: @"noMagField"]];
    }
    else
    {
        [item setLabel: @"Enable Field"];
        [item setToolTip: @"Enable the ambient magnetic field"];
        [item setImage:[NSImage imageNamed: @"magField"]];
    }
}

- (void)setupToolbarItem:(NSToolbarItem *)item
{
    NSString *identifier = [item itemIdentifier];
    if ([identifier isEqual: PauseTBI])
    {
        [self setupPauseToolbarItem: item
                           forState: [model updateIsActive]];
    }
    else if ([identifier isEqual: FieldTBI])
    {
        [self setupFieldToolbarItem: item
                           forState: [model fieldIsPresent]];
    }
}

- (NSToolbarItem *)toolbar:(NSToolbar *)toolbar
     itemForItemIdentifier:(NSString *)itemIdent
 willBeInsertedIntoToolbar:(BOOL)flag
{
    NSToolbarItem *toolbarItem = [[NSToolbarItem alloc] initWithItemIdentifier: itemIdent];

    [toolbarItem autorelease];
    if ([itemIdent isEqual: StatisticsTBI])
    {
        [toolbarItem setPaletteLabel: @"Show/Hide Statistics"];
        [toolbarItem setLabel: @"Statistics"];
        [toolbarItem setToolTip: @"Show or Hide the Statistics drawer"];
        [toolbarItem setImage: [NSImage imageNamed: @"statistics"]];
        [toolbarItem setTarget: self];
        [toolbarItem setAction: @selector(statsDrawerAction:)];
    }
    else if ([itemIdent isEqual: PauseTBI])
    {       
        [toolbarItem setPaletteLabel: @"Pause/Resume"];
        [toolbarItem setImage: [NSImage imageNamed: @"pause"]];
        [toolbarItem setTarget: self];
        [toolbarItem setAction: @selector(pauseButtonAction:)];
    }
    else if ([itemIdent isEqual: FieldTBI])
    {
        [toolbarItem setPaletteLabel: @"Enable/Disable Magnetic Field"];
        [toolbarItem setImage: [NSImage imageNamed: @"magField"]];
        [toolbarItem setTarget: self];
        [toolbarItem setAction: @selector(toggleMagneticFieldAction:)];
    }
    else if ([itemIdent isEqual: UpdateRateTBI])
    {
        [toolbarItem setPaletteLabel: @"Control Update Rate"];
        [toolbarItem setLabel: @"Update Rate"];
        [toolbarItem setToolTip: @"Controls the model update rate"];
        // need to set the size or else it won't appear
        [toolbarItem setMinSize: NSMakeSize(54, 16)];
        [toolbarItem setMaxSize: NSMakeSize(54, 16)];
        // seem to need to retain the view else crashes if user removes it
        [toolbarItem setView: [updateRateSlider retain]];
    }
    else
    {
        NSLog(@"Unknown toolbarItem type: %@", itemIdent);
        return nil;
    }

    return toolbarItem;
}

- (void)toolbarWillAddItem:(NSNotification *)notification
{
    NSToolbarItem *addedItem = [[notification userInfo] objectForKey: @"item"];

    [self setupToolbarItem: addedItem];
}

- (NSArray *)toolbarAllowedItemIdentifiers:(NSToolbar*)toolbar
{
    return [NSArray arrayWithObjects:
        NSToolbarPrintItemIdentifier,
        NSToolbarCustomizeToolbarItemIdentifier,
        NSToolbarFlexibleSpaceItemIdentifier,
        NSToolbarSpaceItemIdentifier,
        NSToolbarSeparatorItemIdentifier,
        FieldTBI,
        StatisticsTBI,
        PauseTBI,
        UpdateRateTBI,
        nil];
}

- (NSArray *)toolbarDefaultItemIdentifiers:(NSToolbar*)toolbar
{
    return [NSArray arrayWithObjects:
        FieldTBI,
        StatisticsTBI,
        PauseTBI,
        UpdateRateTBI,
        NSToolbarFlexibleSpaceItemIdentifier,
        nil];
}

- (BOOL)validateToolbarItem:(NSToolbarItem *)theItem
{
    BOOL enable = YES;

    //if ([[theItem itemIdentifier] isEqual:StatisticsTBI])
    //{
    //    enable = YES;
    //}

    return enable;
}

- (void)updateToolbarItems
{
    NSToolbar *toolbar = [[self window] toolbar];
    NSArray *items = [toolbar items];
    int numItems = [items count];
    int i;
    for (i = 0; i < numItems; i++)
    {
        NSToolbarItem *item = [items objectAtIndex: i];
        [self setupToolbarItem: item];
    }
}

@end

