//
//  IsingViewCtl.m
//  IsingCocoa
//
//  Created by Cameron Hayne on 10/03/05.
//  Copyright Hayne of Tintagel 2005
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

#import "IsingViewCtl.h"

// private methods
@interface IsingViewCtl (Private)
- (NSArray *)viewsForWindow:(NSWindow *)aWindow;
- (void)removeViewsForWindow:(NSNotification *)notification;
@end


@implementation IsingViewCtl

- (id)init
{
    self = [super init];
    if (self == nil)
        return nil;

    //NSLog(@"IsingViewCtl.init");
    myViews = [[NSMutableArray arrayWithCapacity: 10] retain];    
    mySpins = allocate_spins(0, 0);
    
    return self;
}

- (void)dealloc
{
    //NSLog(@"IsingViewCtl.dealloc");
    [myViews removeAllObjects];
    [myViews release];
    free_spins(mySpins);
    [super dealloc];
}

- (void)addView:(id <IsingView>)aView
{
    [myViews addObject: aView];
    
    NSWindow *win = [aView isingViewWindow];
    if (win != nil)
    {
        // ensure that we are notified when the window closes
        // so we can remove this view from our list
        [[NSNotificationCenter defaultCenter]
	addObserver: self
	   selector: @selector(removeViewsForWindow:)
	       name: NSWindowWillCloseNotification
	     object: win];
    }
}

- (void)removeView:(id <IsingView>)aView
{
    [myViews removeObject: aView];
}

- (void)removeViewsForWindow:(NSNotification *)notification
{
    //NSLog(@"IsingViewCtl: removeViewsForWindow");
    NSWindow *winClosing = [notification object];
    [myViews removeObjectsInArray: [self viewsForWindow: winClosing]];
}

- (NSArray *)viewsForWindow:(NSWindow *)aWindow
{
    NSMutableArray *winViews = [NSMutableArray arrayWithCapacity: 10];
    int numViews = [myViews count];
    for (int i = 0; i < numViews; i++)
    {
        id <IsingView> view = [myViews objectAtIndex: i];
        if ([view isingViewWindow] == aWindow)
        {
            [winViews addObject: view];
        }
    }
    //NSLog(@"IsingViewCtl viewsForWindows: %d views", [winViews count]);
    return winViews;
}

#pragma mark Spin methods

- (void)updateIsingViewSpinsSize:(IsingSize)size
{
    int numViews = [myViews count];
    for (int i = 0; i < numViews; i++)
    {
        id <IsingView> view = [myViews objectAtIndex: i];
        [view updateIsingViewSpinsSize: size];
    }
}

// The following method copies the model's spins into the memory previously
// allocated by the view controller for its own copy of the spins
// It is expected that the model will lock the spins array while it is being copied
- (void)updateIsingViewSpins:(Spins *)modelSpins
{
    copy_spins(mySpins, modelSpins);
    
    int numViews = [myViews count];
    for (int i = 0; i < numViews; i++)
    {
        id <IsingView> view = [myViews objectAtIndex: i];
        [view updateIsingViewSpins: mySpins];
    }
}

@end
