//
//  IsingModel.h
//  IsingCocoa
//
//  Created by Cameron Hayne on Tue May 21 2002.
//  Copyright Hayne of Tintagel 2005
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

#import <Foundation/Foundation.h>
#import "ising.h"

extern NSString *IsingDataAvailableNotification;
extern NSString *IsingTemperatureChangedNotification;
extern NSString *IsingFieldChangedNotification;


@interface IsingModel : NSObject
{
    BOOL		fieldIsPresent; // whether a magnetic field is present
    double		temperature;	// parameter set via the Temperature slider
    double		field;		// parameter set by the Magnetic Field slider
    BoltzFactor         boltzUp[5];	// Boltzman factors
    int			myWidth, myHeight; // width = n_cols   height = n_rows
    Spins		*spins;		// current spin configuration
    Spins		*newSpins;	// spin configuration being calculated
    IsingInitState	initState;	// initial configuration state
    int			numSweeps;	// # of lattice sweeps
    float               maxUpdateRate;  // max fps for model update
    double		magnetization;	// latest magnetization value
    double		nnCorrelation;	// latest nearest-neighbour correlation value

    // programming artifacts:
    NSLock		*calculationLock;
    NSLock		*copyingLock;
    volatile BOOL	updateIsActive;
    float               upperBoundModelWidth;
}

// Note on threads & locks:
// We need two separate NSLock variables ('calculationLock' and 'copyingLock')
// because each of the two threads sometimes reads and writes
// the shared variable 'spins'.
// The main ("display") thread reads 'spins' before doing a redraw (it copies the spins
// into the spins array of IsingView)
// and it writes 'spins' when the window is resized.
// The secondary ("calculation") thread reads 'spins' while calculating 'newSpins'
// and overwrites 'spins' with 'newSpins' once the calculation is done.
// We don't allow 'spins' to be read while it is being written to
// or written to while it is being read.
// It is quite likely that at any given time, each thread will hold one of the locks.

- (IsingSize)size;
- (IsingSize)acceptableSize:(IsingSize)proposedSize;
- (IsingSize)requestSize:(IsingSize)aSize modelWasReinit:(BOOL *)reinit;
- (void)initialize;
- (void)setSpinConfig:(IsingInitState)desiredState;
- (BOOL)updateIsActive;
- (void)startUpdate;
- (void)stopUpdate;
- (void)update:(id)obj;
- (Spins *)lockSpinsForDisplay;
- (void)unlockSpinsForDisplay;
- (int)numSweeps;
- (float)maxUpdateRate;
- (void)setMaxUpdateRate:(float)fps;
- (double)temperature;
- (void)setTemperature:(double)value;
- (BOOL)fieldIsPresent;
- (void)setFieldIsPresent:(BOOL)state;
- (double)field;
- (void)setField:(double)value;
- (double)magnetization;
- (double)nnCorrelation;

@end
