//
//  ising.h
//  IsingCocoa
//
//  Created by Cameron Hayne on August 4, 1991.
//  (Modified from code written when I was at Visual Edge Software in 1991)
//  Copyright Hayne of Tintagel 2005
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

#ifndef ISING_H
#define ISING_H

typedef struct
{
    int          width, height; // numCols, numRows
} IsingSize;

typedef unsigned int	SpinWord;

typedef struct
{
    SpinWord    *spinWords;
    int          numRows, numCols;
} Spins;

// WORD_SIZE is the number of bits in a spin_word
#define WORD_SIZE	(8 * sizeof(SpinWord))
#define HIGH_BIT	(1 << (WORD_SIZE-1))

typedef enum { AllSpinsUp, AllSpinsDown, RandomSpins } IsingInitState;


#define SCALE_BOLTZ_TO_MAX_RANDOM 1
/* We scale up the Boltzmann factors by MAX_RANDOM
   and then store them as unsigned longs.
   This allows greater efficiency in comparing with the random numbers. */

#if SCALE_BOLTZ_TO_MAX_RANDOM
typedef unsigned long BoltzFactor;
#else
typedef double BoltzFactor;
#endif

// declarations of functions
extern void     init_ising();
extern void	calc_boltz(double temperature, double field, BoltzFactor *boltz_up);
extern void	initialize_spins(Spins *spins, IsingInitState init_state);
extern void	update_spins(Spins *spins, BoltzFactor boltz_up[5]);
extern void	calc_ising_statistics(const Spins *spins,
                                      double *ptr_magn, double *ptr_nn_corr);
extern void	copy_spins(Spins *destSpins, const Spins *srcSpins);
extern Spins    *allocate_spins(int n_rows, int n_cols);
extern int      num_overflow_spins(int n_cols);
extern inline int      num_spinwords_per_row(int n_cols);
extern inline int      total_num_spinwords(int n_rows, int n_cols);
extern void     free_spins(Spins *spins);
extern int      spinIsUp(Spins *spins, int row, int col);
extern char     *spinWordToString(SpinWord spin);
extern IsingSize calc_block_spins_size(IsingSize origSize, IsingSize blockSize);
extern Spins    *calc_block_spins(Spins *origSpins, IsingSize blockSize);
extern IsingSize makeIsingSize(int width, int height);

#endif // ISING_H

