//
//  randnum.h
//  IsingCocoa
//
//  Created by Cameron Hayne on Wed Jun 05 2002.
//  Copyright Hayne of Tintagel 2005
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

#include <limits.h> // for LONG_MAX

#ifndef SCALE_BOLTZ_TO_MAX_RANDOM
// SCALE_BOLTZ_TO_MAX_RANDOM is defined in ising.h
// which must be included before this file
#error
#endif

// This header file defines the following macros:
// INIT_RAND_NUM()
// RAND_NUM()
// MAX_RAND
// HALF_RAND
// You should make sure that only one of the "USE_xxx" is defined

#define USE_MY_RANDOM 1
// This is a slightly modified version of the standard BSD random()
// It turns out to be much more efficient to have it as part of the program
// instead of in a shared library. But I made a few small optimizations as well.

#define USE_R250_521 0
// See http://www.qbrundage.com/michaelb/pubs/essays/random_number_generation
// This is indeed a lot faster but it has extremely bad randomness
// - there are visually evident dark bands (more spins down) in the initial
// part of an Ising simulation. Seems to get better after a minute or so (?)
// This is probably a problem with this particular implementation
// - perhaps just a problem with the initialization.

#define USE_MTRAND 0
// The Mersenne Twister random number generator
// is supposed to be about twice as fast as random()
// but I don't see that here. It seems to be slower than random()
// so we continue to use random() instead of Mersenne Twister.
// Part of the problem is that
// I don't know how to make it an inline function since
// it is a C++ function being called from C.
// ***** This needs more investigation *****
// Maybe try using the original Mersenne Twister functions - they were in C.
// And try to figure out why the C++ version is so much faster.

// ---------------------------------------------------------------------------/
// Determine which random number generator to use:
#if USE_MTRAND
  #include "MTwrapper.h"
  // the above header file defines MAX_RANDOM for the MTrand_uint32() function
#elif USE_R250_521
  extern void r250_521_init(unsigned long seed);
  extern unsigned long r250_521_random();
  #define MAX_RANDOM	ULONG_MAX
#elif USE_MY_RANDOM
  // be aware that the following statement includes a file of C code!
  #include "MyRandom.c.h"
  #define MAX_RANDOM	LONG_MAX
#else
  #include <stdlib.h>
  #define MAX_RANDOM	LONG_MAX
#endif

#define ONE_OVER_MAX_RANDOM	(1.0 / MAX_RANDOM)

// We need the random numbers to be of the same type as the Boltzmann factors
#if SCALE_BOLTZ_TO_MAX_RANDOM
  // unsigned long random numbers
  #define HALF_MAX (MAX_RANDOM / 2)
  #if USE_MTRAND
    #define INIT_RAND_NUM(seed) (MTsrand_uint32(seed))
    #define RAND_NUM() (MTrand_uint32())
  #elif USE_R250_521
    #define INIT_RAND_NUM(seed) (r250_521_init(seed))
    #define RAND_NUM() (r250_521_random())
  #elif USE_MY_RANDOM
    #define INIT_RAND_NUM(seed) (myInitType4(seed))
    #define RAND_NUM() (myRandom())
  #else
    #define INIT_RAND_NUM(seed) (srandom(seed))
    #define RAND_NUM() (random())
  #endif
#else
  // floating point random numbers
  #define HALF_MAX (0.5)
  #define INIT_RAND_NUM(seed) (srandom(seed))
  #define RAND_NUM() (random() * ONE_OVER_MAX_RANDOM)
#endif
